""" Constants used in the data hierarchy configuration """
#  PASTA-ELN and all its sub-parts are covered by the MIT license.
#
#  Copyright (c) 2023
#
#  Author: Jithu Murugan
#  Filename: constants.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information.

# Data hierarchy table view constants
METADATA_TABLE_LIST_COLUMN_INDEX = 2
METADATA_TABLE_IRI_COLUMN_INDEX = 4
METADATA_TABLE_REQUIRED_COLUMN_INDEX = 5
METADATA_TABLE_DELETE_COLUMN_INDEX = 6
METADATA_TABLE_REORDER_COLUMN_INDEX = 7

ATTACHMENT_TABLE_DELETE_COLUMN_INDEX = 2
ATTACHMENT_TABLE_REORDER_COLUMN_INDEX = 3

# Help page URL
DATA_HIERARCHY_HELP_PAGE_URL = "https://pasta-eln.github.io/pasta-eln/data_hierarchy_configuration.html"
